from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker
from sqlalchemy import Column, Integer, Text, DateTime

from cog_nba.settings import ENGINE

from datetime import datetime

Base = declarative_base()


class Query(Base):
    __tablename__ = 'query'
    id = Column(Integer, primary_key=True, autoincrement=True)
    endpoint = Column(Text)
    params = Column(Text)
    data = Column(Text)
    date = Column(DateTime, default=datetime.utcnow())


class DBConnection(object):

    def __init__(self):
        self.session = None

    def __enter__(self):
        Session = sessionmaker(bind=ENGINE)
        self.session = Session()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.session.close()


def create_tables():
    Base.metadata.create_all(ENGINE)



import json
from datetime import datetime
from requests import Session

from cog_nba.settings import USER_AGENT, MAX_CACHE_AGE
from cog_nba.database import DBConnection, Query


def construct_url(endpoint):
    # Construct URL based on endpoint name.
    # https://github.com/seemethere/nba_py/wiki/stats.nba.com-Endpoint-Documentation
    # Documentation states https://stat.nba.com/stats/<endpoint>/?<params>

    url = 'https://stats.nba.com/stats/%s' % endpoint

    return url


def get(endpoint, *args):
    # Base function for the other functions in this wrapper.
    # Requires endpoint as str and params as dict.

    if type(endpoint) != str:
        raise Exception('Endpoint not a string.')

    payload = {}
    for arg in args:
        if type(arg) != dict:
            raise Exception('Invalid parameter passed, should be dict.')
        else:
            payload.update(arg)

    if check_database(endpoint, json.dumps(payload)):
        # If recent record is found, then return data from SQL rather than excute GET.
        with DBConnection() as db:
            query = db.session.query(Query).filter_by(endpoint=endpoint, params=json.dumps(payload)).first()
            db.session.close()
            return json.loads(query.data)

    else:
        # Rotate user agent and IPs to avoid being blocked
        # TODO: Build in proxies into the GET function.

        headers = {
            'host': 'stats.nba.com',
            'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3',
            'connection': 'keep-alive',
            'user-agent': USER_AGENT,
        }

        url = construct_url(endpoint)

        with Session() as session:
            response = session.get(url, params=payload, headers=headers)

        store_response(endpoint, payload, response)

        return response.json()


def check_database(endpoint, params):
    # Checks table for existing record, and checks how old that record is.
    # MAX_CACHE_AGE can be set in settings

    with DBConnection() as db:
        query = db.session.query(Query).filter_by(endpoint=endpoint, params=params)
        db.session.close()

    if query.scalar():
        if (datetime.utcnow() - query.first().date).days > MAX_CACHE_AGE:
            return False
        else:
            return True
    else:
        return False


def store_response(endpoint, params, response):
    # Store the JSON data captured from the request into a table.
    # Note, params and data should be called with json.dumps() when comparing in check_database().

    response = response.json()

    query = Query(
        endpoint=endpoint,
        params=json.dumps(params),
        data=json.dumps(response)
    )

    with DBConnection() as db:
        try:
            db.session.add(query)
            db.session.commit()
        except Exception as e:
            db.session.rollback()
            print(getattr(e, 'message', repr(e)))


def get_current_players():
    # Shortcut function to get all current players.

    params = {
        'LeagueID': '00',
        'Season': '2018-19',
        'IsOnlyCurrentSeason': '1'
    }

    response = get('commonallplayers', params)

    return response


def get_all_players():
    # Shortcut function to get all players.

    params = {
        'LeagueID': '00',
        'Season': '2018-19',
        'IsOnlyCurrentSeason': '0'
    }

    response = get('commonallplayers', params)

    return response


def get_player_career_stats(player_id, metric):
    # Get player stats, metric should be 'Totals', 'PerGame', 'Per36'

    if str(metric) not in ['Totals', 'PerGame', 'Per36']:
        raise Exception('Metric should be "Totals", "PerGame" or "Per36".')

    try:
        int(player_id)
    except ValueError:
        raise Exception('Player ID is not valid, needs to be a number.')

    params = {
        'PerMode': metric,
        'PlayerID': player_id
    }

    response = get('playercareerstats', params)

    return response



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="cog_nba",
    version="0.0.8",
    author="Andryo Marzuki",
    author_email="stabbish@gmail.com",
    description="A wrapper over the stats.nba.com API which builds in caching of GET queries.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://andryo.co",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'sqlalchemy',
        'requests'
    ]
)



from sqlalchemy import create_engine

# Set up header variables here for GET request, by default we use a fixed agent.
# TODO: Figure out how to rotate the headers without breaking the GET request.

USER_AGENT = 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/73.0.3683.103 Safari/537.36'

# This project uses a SQL database to cache queries to reduce the strain on the API.
# By default, SQLite is used.
# TODO: Switch to PostgreSQL.

ENGINE = create_engine('sqlite:///nba.db', echo=False)
MAX_CACHE_AGE = 7  # Maximum cache age in days
